@@ -42,7 +42,7 @@ from __future__ import with_statement
 import base64, bz2, hashlib, os, random, sys, re, string, tempfile, subprocess, types, functools
 import fabric, fabric.api, fabric.operations, fabric.context_managers
 
-VERSION     = "0.2.6"
+VERSION     = "0.2.7"
 
 RE_SPACES   = re.compile("[\s\t]+")
 MAC_EOL     = "\n"
@@ -354,13 +354,20 @@ def file_is_link(location):
 def file_attribs(location, mode=None, owner=None, group=None, recursive=False):
 	"""Updates the mode/owner/group for the remote file at the given
 	location."""
+	# FIXME: Not sure this is the right way to do it, maybe we should
+	# have USER/SUDO/USER+SUDO modes instead?
+	def run_or_sudo(cmd):
+		with fabric.api.settings(warn_only=True):
+			r = run(cmd)
+			if r.return_code != 0:
+				sudo(cmd)
 	recursive = recursive and "-R " or ""
 	if mode:
-		run('chmod %s %s "%s"' % (recursive, mode,  location))
+		run_or_sudo('chmod %s %s "%s"' % (recursive, mode,  location))
 	if owner:
-		run('chown %s %s "%s"' % (recursive, owner, location))
+		run_or_sudo('chown %s %s "%s"' % (recursive, owner, location))
 	if group:
-		run('chgrp %s %s "%s"' % (recursive, group, location))
+		run_or_sudo('chgrp %s %s "%s"' % (recursive, group, location))
 
 def file_attribs_get(location):
 	"""Return mode, owner, and group for remote path.
@@ -772,6 +779,14 @@ def group_user_ensure(group, user):
 	if user not in d["members"]:
 		group_user_add(group, user)
 
+def group_user_del(group, user):
+        """remove the given user from the given group."""
+        assert group_check(group), "Group does not exist: %s" % (group)
+        if group_user_check(group, user):
+                group_for_user = run("cat /etc/group | egrep -v '^%s:' | grep '%s' | awk -F':' '{print $1}' | grep -v %s; true" % (group, user, user)).splitlines()
+                if group_for_user:
+                        sudo("usermod -G '%s' '%s'" % (",".join(group_for_user), user))
+
 ### ssh_<operation> functions
 
 def ssh_keygen(user, keytype="dsa"):
